@@ -1,10 +1,8 @@
-#!/usr/bin/env python
-from __future__ import absolute_import, division, print_function, unicode_literals
 # -*- coding: utf-8 -*-
 """python library for accessing the openHAB REST API"""
 
 #
-# Georges Toth (c) 2016 <georges@trypill.org>
+# Georges Toth (c) 2016-present <georges@trypill.org>
 #
 # python-openhab is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,31 +20,34 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 # pylint: disable=bad-indentation
 
-import six
-import datetime
+import typing
+
 import dateutil.parser
-from openhab.types import *
+
+import openhab.types
 
 __author__ = 'Georges Toth <georges@trypill.org>'
 __license__ = 'AGPLv3+'
 
 
-class Item(object):
+class Item:
   """Base item class"""
-  def __init__(self, openhab, json_data):
+  types = []  # type: typing.List[typing.Type[openhab.types.CommandType]]
+
+  def __init__(self, openhab_conn: 'openhab.client.OpenHAB', json_data: dict) -> None:
     """
     Args:
-      openhab (openHAB): openHAB object.
+      openhab_conn (openhab.OpenHAB): openHAB object.
       json_data (dic): A dict converted from the JSON data returned by the openHAB
                        server.
     """
-    self.openhab = openhab
+    self.openhab = openhab_conn
     self.type_ = None
     self.name = ''
-    self._state = None
+    self._state = None  # type: typing.Optional[typing.Any]
     self.init_from_json(json_data)
 
-  def init_from_json(self, json_data):
+  def init_from_json(self, json_data: dict):
     """Initialize this object from a json configuration as fetched from
     openHAB
 
@@ -59,7 +60,7 @@ class Item(object):
     self.__set_state(json_data['state'])
 
   @property
-  def state(self):
+  def state(self) -> typing.Any:
     """The state property represents the current state of the item. The state is
     automatically refreshed from openHAB on reading it.
     Updating the value via this property send an update to the event bus.
@@ -70,41 +71,51 @@ class Item(object):
     return self._state
 
   @state.setter
-  def state(self, value):
+  def state(self, value: typing.Any):
     self.update(value)
 
-  def _validate_value(self, value):
+  def _validate_value(self, value: typing.Union[str, typing.Type[openhab.types.CommandType]]):
     """Private method for verifying the new value before modifying the state of the
     item.
     """
     if self.type_ == 'String':
-      if not isinstance(value, six.string_types):
+      if not isinstance(value, str):
         raise ValueError()
-    elif 'types' in dir(self):
+    elif self.types:
+      validation = False
+
       for type_ in self.types:
-        type_.validate(value)
+        try:
+          type_.validate(value)
+        except ValueError:
+          pass
+        else:
+          validation = True
+
+      if not validation:
+        raise ValueError('Invalid value "{}"'.format(value))
     else:
       raise ValueError()
 
-  def _parse_rest(self, value):
+  def _parse_rest(self, value: str) -> str:
     """Parse a REST result into a native object."""
     return value
 
-  def _rest_format(self, value):
+  def _rest_format(self, value: str) -> str:
     """Format a value before submitting to openHAB."""
     return value
 
-  def __set_state(self, value):
+  def __set_state(self, value: str):
     """Private method for setting the internal state."""
     if value in ('UNDEF', 'NULL'):
       self._state = None
     else:
       self._state = self._parse_rest(value)
 
-  def __str__(self):
+  def __str__(self) -> str:
     return '<{0} - {1} : {2}>'.format(self.type_, self.name, self._state)
 
-  def update(self, value):
+  def update(self, value: typing.Any):
     """Updates the state of an item.
 
     Args:
@@ -115,9 +126,11 @@ class Item(object):
 
     v = self._rest_format(value)
 
+    # noinspection PyTypeChecker
     self.openhab.req_put('/items/' + self.name + '/state', data=v)
 
-  def command(self, value):
+  # noinspection PyTypeChecker
+  def command(self, value: typing.Any):
     """Sends the given value as command to the event bus.
 
     Args:
@@ -133,7 +146,7 @@ class Item(object):
 
 class DateTimeItem(Item):
   """DateTime item type"""
-  types = [DateTimeType]
+  types = [openhab.types.DateTimeType]
 
   def __gt__(self, other):
     return self._state > other
@@ -173,20 +186,20 @@ class DateTimeItem(Item):
 
 class SwitchItem(Item):
   """SwitchItem item type"""
-  types = [OnOffType]
+  types = [openhab.types.OnOffType]
 
   def on(self):
     """Set the state of the switch to ON"""
-    self.state = 'ON'
+    self.command('ON')
 
   def off(self):
     """Set the state of the switch to OFF"""
-    self.state = 'OFF'
+    self.command('OFF')
 
 
 class NumberItem(Item):
   """NumberItem item type"""
-  types = [DecimalType]
+  types = [openhab.types.DecimalType]
 
   def _parse_rest(self, value):
     """Parse a REST result into a native object
@@ -213,7 +226,7 @@ class NumberItem(Item):
 
 class ContactItem(Item):
   """Contact item type"""
-  types = [OpenCloseType]
+  types = [openhab.types.OpenCloseType]
 
   def open(self):
     """Set the state of the contact item to OPEN"""
@@ -222,3 +235,138 @@ class ContactItem(Item):
   def closed(self):
     """Set the state of the contact item to CLOSED"""
     self.state = 'CLOSED'
+
+
+class DimmerItem(Item):
+  """DimmerItem item type"""
+  types = [openhab.types.OnOffType, openhab.types.PercentType, openhab.types.IncreaseDecreaseType]
+
+  def _parse_rest(self, value):
+    """Parse a REST result into a native object
+
+    Args:
+      value (str): A string argument to be converted into a int object.
+
+    Returns:
+      int: The int object as converted from the string parameter.
+    """
+    return int(float(value))
+
+  def _rest_format(self, value: typing.Any):
+    """Format a value before submitting to openHAB
+
+    Args:
+      value: Either a string or an integer; in the latter case we have to cast it to a string.
+
+    Returns:
+      str: The string as possibly converted from the parameter.
+    """
+    if not isinstance(value, str):
+      return str(value)
+
+    return value
+
+  def on(self):
+    """Set the state of the dimmer to ON"""
+    self.command('ON')
+
+  def off(self):
+    """Set the state of the dimmer to OFF"""
+    self.command('OFF')
+
+  def increase(self):
+    """Increase the state of the dimmer"""
+    self.command('INCREASE')
+
+  def decrease(self):
+    """Decrease the state of the dimmer"""
+    self.command('DECREASE')
+
+
+class ColorItem(Item):
+  """ColorItem item type"""
+  types = [openhab.types.OnOffType, openhab.types.PercentType, openhab.types.IncreaseDecreaseType,
+           openhab.types.ColorType]
+
+  def _parse_rest(self, value):
+    """Parse a REST result into a native object
+
+    Args:
+      value (str): A string argument to be converted into a str object.
+
+    Returns:
+      str: The str object as converted from the string parameter.
+    """
+    return str(value)
+
+  def _rest_format(self, value: typing.Any):
+    """Format a value before submitting to openHAB
+
+    Args:
+      value: Either a string or an integer; in the latter case we have to cast it to a string.
+
+    Returns:
+      str: The string as possibly converted from the parameter.
+    """
+    if not isinstance(value, str):
+      return str(value)
+
+    return value
+
+  def on(self):
+    """Set the state of the color to ON"""
+    self.command('ON')
+
+  def off(self):
+    """Set the state of the color to OFF"""
+    self.command('OFF')
+
+  def increase(self):
+    """Increase the state of the color"""
+    self.command('INCREASE')
+
+  def decrease(self):
+    """Decrease the state of the color"""
+    self.command('DECREASE')
+
+
+class RollershutterItem(Item):
+  """RollershutterItem item type"""
+  types = [openhab.types.UpDownType, openhab.types.PercentType, openhab.types.StopType]
+
+  def _parse_rest(self, value):
+    """Parse a REST result into a native object
+
+    Args:
+      value (str): A string argument to be converted into a int object.
+
+    Returns:
+      int: The int object as converted from the string parameter.
+    """
+    return int(float(value))
+
+  def _rest_format(self, value: typing.Any):
+    """Format a value before submitting to openHAB
+
+    Args:
+      value: Either a string or an integer; in the latter case we have to cast it to a string.
+
+    Returns:
+      str: The string as possibly converted from the parameter.
+    """
+    if not isinstance(value, str):
+      return str(value)
+
+    return value
+
+  def up(self):
+    """Set the state of the dimmer to ON"""
+    self.command('UP')
+
+  def down(self):
+    """Set the state of the dimmer to OFF"""
+    self.command('DOWN')
+
+  def stop(self):
+    """Set the state of the dimmer to OFF"""
+    self.command('STOP')
